searchData={"items":[{"type":"module","title":"AbnfParsec","doc":"Generates a parser from ABNF definition - text (`:abnf`) or file path (`:abnf_file`)\n\nAn entry rule can be defined by `:parse`. If defined, a `parse/1` function and a\n`parse!/1` function will be generated with the entry rule.\n\nBy default, every chunk defined by a rule is wrapped (in list) and tagged by the\nrulename. Use the options to `:untag`, `:unwrap` or both (`:unbox`).\n\nParsed chunks (rules) can be discarded `:ignore`.\n\nTransformations (`:map`, `:reduce`, `:replace`) can be applied by passing in a\n`:transform` map with keys being rulenames and values being 2-tuples of\ntransformation type (`:map`, `:reduce`, `:replace`), and mfa tuple (for `:map` and `reduce`)\nor a literal value (for `:replace`)\n\nExample usage:\n\n    defmodule JsonParser do\n      use AbnfParsec,\n        abnf_file: \"test/fixture/json.abnf\",\n        parse: :json_text,\n        transform: %{\n          \"string\" => {:reduce, {List, :to_string, []}},\n          \"int\" => [{:reduce, {List, :to_string, []}}, {:map, {String, :to_integer, []}}],\n          \"frac\" => {:reduce, {List, :to_string, []}},\n          \"null\" => {:replace, nil},\n          \"true\" => {:replace, true},\n          \"false\" => {:replace, false}\n        },\n        untag: [\"member\"],\n        unwrap: [\"int\", \"frac\"],\n        unbox: [\n          \"JSON-text\",\n          \"null\",\n          \"true\",\n          \"false\",\n          \"digit1-9\",\n          \"decimal-point\",\n          \"escape\",\n          \"unescaped\",\n          \"char\"\n        ],\n        ignore: [\n          \"name-separator\",\n          \"value-separator\",\n          \"quotation-mark\",\n          \"begin-object\",\n          \"end-object\",\n          \"begin-array\",\n          \"end-array\"\n        ]\n    end\n\n    json = ~s| {\"a\": {\"b\": 1.2, \"c\": [true]}, \"d\": null, \"e\": \"e\\te\"} |\n\n    JsonParser.json_text(json)\n    # or\n    JsonParser.parse(json)\n    # => {:ok, ...}\n    JsonParser.parse!(json)\n    # =>\n\n    [\n      object: [\n        [\n          string: [\"a\"],\n          value: [\n            object: [\n              [string: [\"b\"], value: [number: [int: 1, frac: \".2\"]]],\n              [string: [\"c\"], value: [array: [value: [true]]]]\n            ]\n          ]\n        ],\n        [string: [\"d\"], value: [nil]],\n        [string: [\"e\"], value: [string: [\"e\\te\"]]]\n      ]\n    ]","ref":"AbnfParsec.html"},{"type":"macro","title":"AbnfParsec.__using__/1","doc":"All rules by default are wrapped and tagged. See NimbleParsec for more details.\n\nOptions:\n\n  - `:abnf` (`binary`) - ABNF directly in string data\n  - `:abnf_file` (`binary`) - ABNF file path\n  - `:debug` (`boolean`) - whether to output generated parser code\n\n  - `:skip` (`rulenames`) - rules to be skipped when generating parser code\n    - so user can define their own parsec definition\n  - `:ignore` (`rulenames`) - rules to be discarded after being parsed\n  - `:untag` (`rulenames`) - rules to be wrapped only after being parsed\n  - `:unwrap` (`rulenames`) - rules to be tagged only after being parsed\n    - needs to be sure that there is only singular parsed entry\n  - `:unbox` (`rulenames`) - made up term, to both untag and unwrap\n  - `:transform` (`transformations`) - a map of `rulenames` to transformations\n    - `{:map, mfa}`\n    - `{:reduce, mfa}`\n    - `{:replace, val}`\n    - `{:post_traverse, mfa}`\n    - `{:pre_traverse, mfa}`\n  - `:parse` (`atom`) - described in module doc, needs to be an atom that is in\n    normalized form of its original string rulename","ref":"AbnfParsec.html#__using__/1"},{"type":"type","title":"AbnfParsec.rulename/0","doc":"","ref":"AbnfParsec.html#t:rulename/0"},{"type":"type","title":"AbnfParsec.rulenames/0","doc":"","ref":"AbnfParsec.html#t:rulenames/0"},{"type":"type","title":"AbnfParsec.transformation/0","doc":"","ref":"AbnfParsec.html#t:transformation/0"},{"type":"type","title":"AbnfParsec.transformations/0","doc":"","ref":"AbnfParsec.html#t:transformations/0"},{"type":"module","title":"AbnfParsec.Parser","doc":"Abnf Parser.","ref":"AbnfParsec.Parser.html"},{"type":"function","title":"AbnfParsec.Parser.alternation/2","doc":"Parses the given `binary` as alternation.\n\nReturns `{:ok, [token], rest, context, position, byte_offset}` or\n`{:error, reason, rest, context, line, byte_offset}` where `position`\ndescribes the location of the alternation (start position) as `{line, offset_to_start_of_line}`.\n\nTo column where the error occurred can be inferred from `byte_offset - offset_to_start_of_line`.","ref":"AbnfParsec.Parser.html#alternation/2"},{"type":"function","title":"Options - AbnfParsec.Parser.alternation/2","doc":"* `:byte_offset` - the byte offset for the whole binary, defaults to 0\n  * `:line` - the line and the byte offset into that line, defaults to `{1, byte_offset}`\n  * `:context` - the initial context value. It will be converted to a map","ref":"AbnfParsec.Parser.html#alternation/2-options"},{"type":"function","title":"AbnfParsec.Parser.char_val/2","doc":"Parses the given `binary` as char_val.\n\nReturns `{:ok, [token], rest, context, position, byte_offset}` or\n`{:error, reason, rest, context, line, byte_offset}` where `position`\ndescribes the location of the char_val (start position) as `{line, offset_to_start_of_line}`.\n\nTo column where the error occurred can be inferred from `byte_offset - offset_to_start_of_line`.","ref":"AbnfParsec.Parser.html#char_val/2"},{"type":"function","title":"Options - AbnfParsec.Parser.char_val/2","doc":"* `:byte_offset` - the byte offset for the whole binary, defaults to 0\n  * `:line` - the line and the byte offset into that line, defaults to `{1, byte_offset}`\n  * `:context` - the initial context value. It will be converted to a map","ref":"AbnfParsec.Parser.html#char_val/2-options"},{"type":"function","title":"AbnfParsec.Parser.comment/2","doc":"Parses the given `binary` as comment.\n\nReturns `{:ok, [token], rest, context, position, byte_offset}` or\n`{:error, reason, rest, context, line, byte_offset}` where `position`\ndescribes the location of the comment (start position) as `{line, offset_to_start_of_line}`.\n\nTo column where the error occurred can be inferred from `byte_offset - offset_to_start_of_line`.","ref":"AbnfParsec.Parser.html#comment/2"},{"type":"function","title":"Options - AbnfParsec.Parser.comment/2","doc":"* `:byte_offset` - the byte offset for the whole binary, defaults to 0\n  * `:line` - the line and the byte offset into that line, defaults to `{1, byte_offset}`\n  * `:context` - the initial context value. It will be converted to a map","ref":"AbnfParsec.Parser.html#comment/2-options"},{"type":"function","title":"AbnfParsec.Parser.concatenation/2","doc":"Parses the given `binary` as concatenation.\n\nReturns `{:ok, [token], rest, context, position, byte_offset}` or\n`{:error, reason, rest, context, line, byte_offset}` where `position`\ndescribes the location of the concatenation (start position) as `{line, offset_to_start_of_line}`.\n\nTo column where the error occurred can be inferred from `byte_offset - offset_to_start_of_line`.","ref":"AbnfParsec.Parser.html#concatenation/2"},{"type":"function","title":"Options - AbnfParsec.Parser.concatenation/2","doc":"* `:byte_offset` - the byte offset for the whole binary, defaults to 0\n  * `:line` - the line and the byte offset into that line, defaults to `{1, byte_offset}`\n  * `:context` - the initial context value. It will be converted to a map","ref":"AbnfParsec.Parser.html#concatenation/2-options"},{"type":"function","title":"AbnfParsec.Parser.core_rule/2","doc":"Parses the given `binary` as core_rule.\n\nReturns `{:ok, [token], rest, context, position, byte_offset}` or\n`{:error, reason, rest, context, line, byte_offset}` where `position`\ndescribes the location of the core_rule (start position) as `{line, offset_to_start_of_line}`.\n\nTo column where the error occurred can be inferred from `byte_offset - offset_to_start_of_line`.","ref":"AbnfParsec.Parser.html#core_rule/2"},{"type":"function","title":"Options - AbnfParsec.Parser.core_rule/2","doc":"* `:byte_offset` - the byte offset for the whole binary, defaults to 0\n  * `:line` - the line and the byte offset into that line, defaults to `{1, byte_offset}`\n  * `:context` - the initial context value. It will be converted to a map","ref":"AbnfParsec.Parser.html#core_rule/2-options"},{"type":"function","title":"AbnfParsec.Parser.element/2","doc":"Parses the given `binary` as element.\n\nReturns `{:ok, [token], rest, context, position, byte_offset}` or\n`{:error, reason, rest, context, line, byte_offset}` where `position`\ndescribes the location of the element (start position) as `{line, offset_to_start_of_line}`.\n\nTo column where the error occurred can be inferred from `byte_offset - offset_to_start_of_line`.","ref":"AbnfParsec.Parser.html#element/2"},{"type":"function","title":"Options - AbnfParsec.Parser.element/2","doc":"* `:byte_offset` - the byte offset for the whole binary, defaults to 0\n  * `:line` - the line and the byte offset into that line, defaults to `{1, byte_offset}`\n  * `:context` - the initial context value. It will be converted to a map","ref":"AbnfParsec.Parser.html#element/2-options"},{"type":"function","title":"AbnfParsec.Parser.exception/2","doc":"Extension: Used in RFC3501","ref":"AbnfParsec.Parser.html#exception/2"},{"type":"function","title":"AbnfParsec.Parser.group/2","doc":"Parses the given `binary` as group.\n\nReturns `{:ok, [token], rest, context, position, byte_offset}` or\n`{:error, reason, rest, context, line, byte_offset}` where `position`\ndescribes the location of the group (start position) as `{line, offset_to_start_of_line}`.\n\nTo column where the error occurred can be inferred from `byte_offset - offset_to_start_of_line`.","ref":"AbnfParsec.Parser.html#group/2"},{"type":"function","title":"Options - AbnfParsec.Parser.group/2","doc":"* `:byte_offset` - the byte offset for the whole binary, defaults to 0\n  * `:line` - the line and the byte offset into that line, defaults to `{1, byte_offset}`\n  * `:context` - the initial context value. It will be converted to a map","ref":"AbnfParsec.Parser.html#group/2-options"},{"type":"function","title":"AbnfParsec.Parser.normalize/1","doc":"","ref":"AbnfParsec.Parser.html#normalize/1"},{"type":"function","title":"AbnfParsec.Parser.num_val/2","doc":"Parses the given `binary` as num_val.\n\nReturns `{:ok, [token], rest, context, position, byte_offset}` or\n`{:error, reason, rest, context, line, byte_offset}` where `position`\ndescribes the location of the num_val (start position) as `{line, offset_to_start_of_line}`.\n\nTo column where the error occurred can be inferred from `byte_offset - offset_to_start_of_line`.","ref":"AbnfParsec.Parser.html#num_val/2"},{"type":"function","title":"Options - AbnfParsec.Parser.num_val/2","doc":"* `:byte_offset` - the byte offset for the whole binary, defaults to 0\n  * `:line` - the line and the byte offset into that line, defaults to `{1, byte_offset}`\n  * `:context` - the initial context value. It will be converted to a map","ref":"AbnfParsec.Parser.html#num_val/2-options"},{"type":"function","title":"AbnfParsec.Parser.option/2","doc":"Parses the given `binary` as option.\n\nReturns `{:ok, [token], rest, context, position, byte_offset}` or\n`{:error, reason, rest, context, line, byte_offset}` where `position`\ndescribes the location of the option (start position) as `{line, offset_to_start_of_line}`.\n\nTo column where the error occurred can be inferred from `byte_offset - offset_to_start_of_line`.","ref":"AbnfParsec.Parser.html#option/2"},{"type":"function","title":"Options - AbnfParsec.Parser.option/2","doc":"* `:byte_offset` - the byte offset for the whole binary, defaults to 0\n  * `:line` - the line and the byte offset into that line, defaults to `{1, byte_offset}`\n  * `:context` - the initial context value. It will be converted to a map","ref":"AbnfParsec.Parser.html#option/2-options"},{"type":"function","title":"AbnfParsec.Parser.parse/1","doc":"","ref":"AbnfParsec.Parser.html#parse/1"},{"type":"function","title":"AbnfParsec.Parser.parse!/1","doc":"","ref":"AbnfParsec.Parser.html#parse!/1"},{"type":"function","title":"AbnfParsec.Parser.prose_val/2","doc":"Parses the given `binary` as prose_val.\n\nReturns `{:ok, [token], rest, context, position, byte_offset}` or\n`{:error, reason, rest, context, line, byte_offset}` where `position`\ndescribes the location of the prose_val (start position) as `{line, offset_to_start_of_line}`.\n\nTo column where the error occurred can be inferred from `byte_offset - offset_to_start_of_line`.","ref":"AbnfParsec.Parser.html#prose_val/2"},{"type":"function","title":"Options - AbnfParsec.Parser.prose_val/2","doc":"* `:byte_offset` - the byte offset for the whole binary, defaults to 0\n  * `:line` - the line and the byte offset into that line, defaults to `{1, byte_offset}`\n  * `:context` - the initial context value. It will be converted to a map","ref":"AbnfParsec.Parser.html#prose_val/2-options"},{"type":"function","title":"AbnfParsec.Parser.repetition/2","doc":"Parses the given `binary` as repetition.\n\nReturns `{:ok, [token], rest, context, position, byte_offset}` or\n`{:error, reason, rest, context, line, byte_offset}` where `position`\ndescribes the location of the repetition (start position) as `{line, offset_to_start_of_line}`.\n\nTo column where the error occurred can be inferred from `byte_offset - offset_to_start_of_line`.","ref":"AbnfParsec.Parser.html#repetition/2"},{"type":"function","title":"Options - AbnfParsec.Parser.repetition/2","doc":"* `:byte_offset` - the byte offset for the whole binary, defaults to 0\n  * `:line` - the line and the byte offset into that line, defaults to `{1, byte_offset}`\n  * `:context` - the initial context value. It will be converted to a map","ref":"AbnfParsec.Parser.html#repetition/2-options"},{"type":"function","title":"AbnfParsec.Parser.rule/2","doc":"Parses the given `binary` as rule.\n\nReturns `{:ok, [token], rest, context, position, byte_offset}` or\n`{:error, reason, rest, context, line, byte_offset}` where `position`\ndescribes the location of the rule (start position) as `{line, offset_to_start_of_line}`.\n\nTo column where the error occurred can be inferred from `byte_offset - offset_to_start_of_line`.","ref":"AbnfParsec.Parser.html#rule/2"},{"type":"function","title":"Options - AbnfParsec.Parser.rule/2","doc":"* `:byte_offset` - the byte offset for the whole binary, defaults to 0\n  * `:line` - the line and the byte offset into that line, defaults to `{1, byte_offset}`\n  * `:context` - the initial context value. It will be converted to a map","ref":"AbnfParsec.Parser.html#rule/2-options"},{"type":"function","title":"AbnfParsec.Parser.rulelist/2","doc":"Parses the given `binary` as rulelist.\n\nReturns `{:ok, [token], rest, context, position, byte_offset}` or\n`{:error, reason, rest, context, line, byte_offset}` where `position`\ndescribes the location of the rulelist (start position) as `{line, offset_to_start_of_line}`.\n\nTo column where the error occurred can be inferred from `byte_offset - offset_to_start_of_line`.","ref":"AbnfParsec.Parser.html#rulelist/2"},{"type":"function","title":"Options - AbnfParsec.Parser.rulelist/2","doc":"* `:byte_offset` - the byte offset for the whole binary, defaults to 0\n  * `:line` - the line and the byte offset into that line, defaults to `{1, byte_offset}`\n  * `:context` - the initial context value. It will be converted to a map","ref":"AbnfParsec.Parser.html#rulelist/2-options"},{"type":"function","title":"AbnfParsec.Parser.rulename/2","doc":"Parses the given `binary` as rulename.\n\nReturns `{:ok, [token], rest, context, position, byte_offset}` or\n`{:error, reason, rest, context, line, byte_offset}` where `position`\ndescribes the location of the rulename (start position) as `{line, offset_to_start_of_line}`.\n\nTo column where the error occurred can be inferred from `byte_offset - offset_to_start_of_line`.","ref":"AbnfParsec.Parser.html#rulename/2"},{"type":"function","title":"Options - AbnfParsec.Parser.rulename/2","doc":"* `:byte_offset` - the byte offset for the whole binary, defaults to 0\n  * `:line` - the line and the byte offset into that line, defaults to `{1, byte_offset}`\n  * `:context` - the initial context value. It will be converted to a map","ref":"AbnfParsec.Parser.html#rulename/2-options"},{"type":"exception","title":"AbnfParsec.LeftoverTokenError","doc":"","ref":"AbnfParsec.LeftoverTokenError.html"},{"type":"exception","title":"AbnfParsec.UnexpectedTokenError","doc":"","ref":"AbnfParsec.UnexpectedTokenError.html"},{"type":"extras","title":"CHANGELOG","doc":"","ref":"changelog.html"},{"type":"extras","title":"v1.3.0 - 2025-01-30 - CHANGELOG","doc":"- Add byte mode to generate parsers that work on byte representation instead of text codepoints","ref":"changelog.html#v1-3-0-2025-01-30"},{"type":"extras","title":"v1.2.6 - 2025-01-29 - CHANGELOG","doc":"- support utf-8 codepoint sequence","ref":"changelog.html#v1-2-6-2025-01-29"},{"type":"extras","title":"v1.2.5 - 2025-01-29 - CHANGELOG","doc":"","ref":"changelog.html#v1-2-5-2025-01-29"},{"type":"extras","title":"🐛 Bug Fixes - CHANGELOG","doc":"- Fix core rule HEXDIG to allow lowercase chars","ref":"changelog.html#bug-fixes"},{"type":"extras","title":"v1.2.4 - 2025-01-28 - CHANGELOG","doc":"","ref":"changelog.html#v1-2-4-2025-01-28"},{"type":"extras","title":"🐛 Bug Fixes - CHANGELOG","doc":"- Fix nimble parsec deprecation warnings @sax (#82)","ref":"changelog.html#bug-fixes"},{"type":"extras","title":"v1.2.3 - 2025-01-06 - CHANGELOG","doc":"","ref":"changelog.html#v1-2-3-2025-01-06"},{"type":"extras","title":"Fixed - CHANGELOG","doc":"- Generate utf8 matcher instead of ascii on num_range (c40c2b9) fix [#76](https://github.com/princemaple/abnf_parsec/issues/76)","ref":"changelog.html#fixed"},{"type":"extras","title":"v1.2.1 - 2022-02-05 - CHANGELOG","doc":"","ref":"changelog.html#v1-2-1-2022-02-05"},{"type":"extras","title":"Fixed - CHANGELOG","doc":"- String concatenation bug caused by case insensitivity implementation","ref":"changelog.html#fixed"},{"type":"extras","title":"v1.2.0 - 2021-03-17 - CHANGELOG","doc":"","ref":"changelog.html#v1-2-0-2021-03-17"},{"type":"extras","title":"Changed - CHANGELOG","doc":"Now strings default to be case insensitive","ref":"changelog.html#changed"},{"type":"extras","title":"v1.1.0 - 2020-12-28 - CHANGELOG","doc":"","ref":"changelog.html#v1-1-0-2020-12-28"},{"type":"extras","title":"🚀 Features - CHANGELOG","doc":"- Rfc7405 @guenni68 (#16)\n  - Adds case-insensitive string matching but haven't switched default to case-insensitive yet","ref":"changelog.html#features"},{"type":"extras","title":"v1.0.0 - 2020-11-03 - CHANGELOG","doc":"Not much, just that it's stable enough\n\n- handle 0 repeat @princemaple (#15)","ref":"changelog.html#v1-0-0-2020-11-03"},{"type":"extras","title":"v0.1.2 - 2020-03-06 - CHANGELOG","doc":"","ref":"changelog.html#v0-1-2-2020-03-06"},{"type":"extras","title":"Added - CHANGELOG","doc":"- `pre/post_traverse` transformation","ref":"changelog.html#added"},{"type":"extras","title":"v0.1.1 - 2020-02-10 - CHANGELOG","doc":"","ref":"changelog.html#v0-1-1-2020-02-10"},{"type":"extras","title":"Changed - CHANGELOG","doc":"- Require Elixir 1.10","ref":"changelog.html#changed"},{"type":"extras","title":"Added - CHANGELOG","doc":"- Allow adding extra UTF8 range to comments\n  - `test/fixture/dhall.abnf` has this line `; \"∀\" / \"forall\"`","ref":"changelog.html#added"},{"type":"extras","title":"v0.1.0 - 2020-02-09 - CHANGELOG","doc":"Initial release","ref":"changelog.html#v0-1-0-2020-02-09"},{"type":"extras","title":"AbnfParsec","doc":"# AbnfParsec\n\n[![hex.pm](https://img.shields.io/hexpm/v/abnf_parsec.svg)](https://hex.pm/packages/abnf_parsec)\n[![hex.pm](https://img.shields.io/hexpm/dt/abnf_parsec.svg)](https://hex.pm/packages/abnf_parsec)\n[![hex.pm](https://img.shields.io/hexpm/l/abnf_parsec.svg)](https://hex.pm/packages/abnf_parsec)\n[![github.com](https://img.shields.io/github/last-commit/princemaple/abnf_parsec.svg)](https://github.com/princemaple/abnf_parsec)","ref":"readme.html"},{"type":"extras","title":"ABNF in and parser out. - AbnfParsec","doc":"Parses ABNF with a parser written with `nimble_parsec`, emits parser consists of `nimble_parsec` combinators.","ref":"readme.html#abnf-in-and-parser-out"},{"type":"extras","title":"Features - AbnfParsec","doc":"- Brevity - flattens unnecessary nesting in parsed ABNF\n- Easy to config and customize\n- Full test coverage","ref":"readme.html#features"},{"type":"extras","title":"Installation - AbnfParsec","doc":"The [package](https://hex.pm/packages/abnf_parsec) is available on [Hex](https://hex.pm)\nand can be installed by adding `abnf_parsec` to your list of dependencies in `mix.exs`:\n\n```elixir\ndef deps do\n  [\n    {:abnf_parsec, \"~> 1.0\", runtime: false}\n  ]\nend\n```\n\nDocumentation can be generated with [ExDoc](https://github.com/elixir-lang/ex_doc)\nand published on [HexDocs](https://hexdocs.pm). Once published, the docs can\nbe found at [https://hexdocs.pm/abnf_parsec](https://hexdocs.pm/abnf_parsec).","ref":"readme.html#installation"},{"type":"extras","title":"Text / Byte mode - AbnfParsec","doc":"In some RFCs, literals in ABNF could be used to describe the byte representation instead of\nthe text codepoints. There is no clear distinction between them for us to detect automatically.\nHence a text / byte mode is added for the user to set.\n\n```elixir\ndefmodule TextParser do\n  use AbnfParsec,\n    mode: :text, # the default, can be omitted\n    abnf: \"\"\"\n    ucschar = %xA0-D7FF / %xF900-FDCF / %xFDF0-FFEF\n      / %x10000-1FFFD / %x20000-2FFFD / %x30000-3FFFD\n      / %x40000-4FFFD / %x50000-5FFFD / %x60000-6FFFD\n      / %x70000-7FFFD / %x80000-8FFFD / %x90000-9FFFD\n      / %xA0000-AFFFD / %xB0000-BFFFD / %xC0000-CFFFD\n      / %xD0000-DFFFD / %xE1000-EFFFD\n    \"\"\"\nend\n\ndefmodule ByteParser do\n  use AbnfParsec,\n    mode: :byte,\n    abnf: \"\"\"\n    ; from RFC 5322 + UTF8-non-ascii\n    atext = ALPHA / DIGIT / \"!\" / \"#\" / \"$\" / \"%\" / \"&\" / \"'\" / \"*\" / \"+\" /\n      \"-\" / \"/\" / \"=\" / \"?\" / \"^\" / \"_\" / \"`\" / \"{\" / \"|\" / \"}\" / \"~\" / UTF8-non-ascii\n\n    ; from RFC 3629\n    UTF8-non-ascii  =   UTF8-2 / UTF8-3 / UTF8-4\n\n    UTF8-2      = %xC2-DF UTF8-tail\n    UTF8-3      = %xE0 %xA0-BF UTF8-tail / %xE1-EC 2( UTF8-tail ) /\n          %xED %x80-9F UTF8-tail / %xEE-EF 2( UTF8-tail )\n    UTF8-4      = %xF0 %x90-BF 2( UTF8-tail ) / %xF1-F3 3( UTF8-tail ) /\n          %xF4 %x80-8F 2( UTF8-tail )\n    UTF8-tail   = %x80-BF\n    \"\"\"\nend\n```","ref":"readme.html#text-byte-mode"},{"type":"extras","title":"Usage - AbnfParsec","doc":"```elixir\ndefmodule IPv4Parser do\n  use AbnfParsec,\n    abnf: \"\"\"\n    ip = first dot second dot third dot fourth\n    dot = \".\"\n    dec-octet =\n      \"25\" %x30-35      /   ; 250-255\n      \"2\" %x30-34 DIGIT /   ; 200-249\n      \"1\" 2DIGIT        /   ; 100-199\n      %x31-39 DIGIT     /   ; 10-99\n      DIGIT                 ; 0-9\n    first = dec-octet\n    second = dec-octet\n    third = dec-octet\n    fourth = dec-octet\n    \"\"\",\n    unbox: [\"dec-octet\"],\n    ignore: [\"dot\"],\n    parse: :ip\nend\n\n# IPv4Parser.ip(\"192.168.0.1\")\n# IPv4Parser.parse(\"127.0.0.1\")\n# IPv4Parser.parse!(\"10.0.0.1\")\n\niex> IPv4Parser.parse! \"10.0.0.1\"\n[ip: [first: '10', second: '0', third: '0', fourth: '1']]\n```\n\n```elixir\ndefmodule JsonParser do\n  use AbnfParsec,\n    abnf_file: \"test/fixture/json.abnf\",\n    parse: :json_text,\n    transform: %{\n      \"string\" => {:reduce, {List, :to_string, []}},\n      \"int\" => [{:reduce, {List, :to_string, []}}, {:map, {String, :to_integer, []}}],\n      \"frac\" => {:reduce, {List, :to_string, []}},\n      \"null\" => {:replace, nil},\n      \"true\" => {:replace, true},\n      \"false\" => {:replace, false}\n    },\n    untag: [\"member\"],\n    unwrap: [\"int\", \"frac\"],\n    unbox: [\n      \"JSON-text\",\n      \"null\",\n      \"true\",\n      \"false\",\n      \"digit1-9\",\n      \"decimal-point\",\n      \"escape\",\n      \"unescaped\",\n      \"char\"\n    ],\n    ignore: [\n      \"name-separator\",\n      \"value-separator\",\n      \"quotation-mark\",\n      \"begin-object\",\n      \"end-object\",\n      \"begin-array\",\n      \"end-array\"\n    ]\nend\n\njson = \"\"\"\n  {\"a\": {\"b\": 1, \"c\": [true]}, \"d\": null, \"e\": \"e\\\\te\"}\n  \"\"\"\n\n# JsonParser.json_text(json)\n# JsonParser.parse(json)\n# JsonParser.parse!(json)\n\niex> JsonParser.parse! \"\"\"\n...> {\"a\": {\"b\": 1, \"c\": [true]}, \"d\": null, \"e\": \"e\\\\te\"}\n...> \"\"\"\n[\n  object: [\n    [\n      string: [\"a\"],\n      value: [\n        object: [\n          [string: [\"b\"], value: [number: [int: 1]]],\n          [string: [\"c\"], value: [array: [value: [true]]]]\n        ]\n      ]\n    ],\n    [string: [\"d\"], value: [nil]],\n    [string: [\"e\"], value: [string: [\"e\\\\te\"]]]\n  ]\n]\n```\n\nFor more details of options for customization, see [abnf_parsec.ex](https://github.com/princemaple/abnf_parsec/blob/main/lib/abnf_parsec.ex)","ref":"readme.html#usage"},{"type":"extras","title":"What does it do, really? - AbnfParsec","doc":"For example, ABNF of ABNF 😉 is parsed\n\n```\nrulelist       =  1*( rule / (*c-wsp c-nl) )\n\nrule           =  rulename defined-as elements c-nl\n                      ; continues if next line starts\n                      ;  with white space\n\nrulename       =  ALPHA *(ALPHA / DIGIT / \"-\")\n\ndefined-as     =  *c-wsp (\"=\" / \"=/\") *c-wsp\n                      ; basic rules definition and\n                      ;  incremental alternatives\n\nelements       =  alternation *c-wsp\n\nc-wsp          =  WSP / (c-nl WSP)\n\nc-nl           =  comment / CRLF\n                      ; comment or newline\n\ncomment        =  \";\" *(WSP / VCHAR) CRLF\n\nalternation    =  concatenation\n                 *(*c-wsp \"/\" *c-wsp concatenation)\n\nconcatenation  =  repetition *(1*c-wsp repetition)\n\nrepetition     =  [repeat] element\n\nrepeat         =  1*DIGIT / (*DIGIT \"*\" *DIGIT)\n\nelement        =  rulename / group / option /\n                 char-val / num-val / prose-val\n\ngroup          =  \"(\" *c-wsp alternation *c-wsp \")\"\n\noption         =  \"[\" *c-wsp alternation *c-wsp \"]\"\n\nchar-val       =  DQUOTE *(%x20-21 / %x23-7E) DQUOTE\n                      ; quoted string of SP and VCHAR\n                      ;  without DQUOTE\n\nnum-val        =  \"%\" (bin-val / dec-val / hex-val)\n\nbin-val        =  \"b\" 1*BIT\n                 [ 1*(\".\" 1*BIT) / (\"-\" 1*BIT) ]\n                      ; series of concatenated bit values\n                      ;  or single ONEOF range\n\ndec-val        =  \"d\" 1*DIGIT\n                 [ 1*(\".\" 1*DIGIT) / (\"-\" 1*DIGIT) ]\n\nhex-val        =  \"x\" 1*HEXDIG\n                 [ 1*(\".\" 1*HEXDIG) / (\"-\" 1*HEXDIG) ]\n\nprose-val      =  \"<\" *(%x20-3D / %x3F-7E) \">\"\n                      ; bracketed string of SP and VCHAR\n                      ;  without angles\n                      ; prose description, to be used as\n                      ;  last resort\n```\n\ninto\n\n```elixir\n[\n  rule: [\n    rulename: \"rulelist\",\n    repetition: [\n      repeat: [min: 1],\n      alternation: [\n        rulename: \"rule\",\n        concatenation: [\n          repetition: [repeat: [], rulename: \"c-wsp\"],\n          rulename: \"c-nl\"\n        ]\n      ]\n    ]\n  ],\n  rule: [\n    rulename: \"rule\",\n    concatenation: [\n      rulename: \"rulename\",\n      rulename: \"defined-as\",\n      rulename: \"elements\",\n      rulename: \"c-nl\"\n    ],\n    comment: \"continues if next line starts\",\n    comment: \"with white space\"\n  ],\n  rule: [\n    rulename: \"rulename\",\n    concatenation: [\n      rulename: \"ALPHA\",\n      repetition: [\n        repeat: [],\n        alternation: [{:rulename, \"ALPHA\"}, {:rulename, \"DIGIT\"}, \"-\"]\n      ]\n    ]\n  ],\n  rule: [\n    rulename: \"defined-as\",\n    concatenation: [\n      repetition: [repeat: [], rulename: \"c-wsp\"],\n      alternation: [\"=\", \"=/\"],\n      repetition: [repeat: [], rulename: \"c-wsp\"]\n    ],\n    comment: \"basic rules definition and\",\n    comment: \"incremental alternatives\"\n  ],\n  rule: [\n    rulename: \"elements\",\n    concatenation: [\n      rulename: \"alternation\",\n      repetition: [repeat: [], rulename: \"c-wsp\"]\n    ]\n  ],\n  rule: [\n    rulename: \"c-wsp\",\n    alternation: [\n      rulename: \"WSP\",\n      concatenation: [rulename: \"c-nl\", rulename: \"WSP\"]\n    ]\n  ],\n  rule: [\n    rulename: \"c-nl\",\n    alternation: [rulename: \"comment\", rulename: \"CRLF\"],\n    comment: \"comment or newline\"\n  ],\n  rule: [\n    rulename: \"comment\",\n    concatenation: [\n      \";\",\n      {:repetition,\n       [repeat: [], alternation: [rulename: \"WSP\", rulename: \"VCHAR\"]]},\n      {:rulename, \"CRLF\"}\n    ]\n  ],\n  rule: [\n    rulename: \"alternation\",\n    concatenation: [\n      rulename: \"concatenation\",\n      repetition: [\n        repeat: [],\n        concatenation: [\n          {:repetition, [repeat: [], rulename: \"c-wsp\"]},\n          \"/\",\n          {:repetition, [repeat: [], rulename: \"c-wsp\"]},\n          {:rulename, \"concatenation\"}\n        ]\n      ]\n    ]\n  ],\n  rule: [\n    rulename: \"concatenation\",\n    concatenation: [\n      rulename: \"repetition\",\n      repetition: [\n        repeat: [],\n        concatenation: [\n          repetition: [repeat: [min: 1], rulename: \"c-wsp\"],\n          rulename: \"repetition\"\n        ]\n      ]\n    ]\n  ],\n  rule: [\n    rulename: \"repetition\",\n    concatenation: [option: [rulename: \"repeat\"], rulename: \"element\"]\n  ],\n  rule: [\n    rulename: \"repeat\",\n    alternation: [\n      repetition: [repeat: [min: 1], rulename: \"DIGIT\"],\n      concatenation: [\n        {:repetition, [repeat: [], rulename: \"DIGIT\"]},\n        \"*\",\n        {:repetition, [repeat: [], rulename: \"DIGIT\"]}\n      ]\n    ]\n  ],\n  rule: [\n    rulename: \"element\",\n    alternation: [\n      rulename: \"rulename\",\n      rulename: \"group\",\n      rulename: \"option\",\n      rulename: \"char-val\",\n      rulename: \"num-val\",\n      rulename: \"prose-val\"\n    ]\n  ],\n  rule: [\n    rulename: \"group\",\n    concatenation: [\n      \"(\",\n      {:repetition, [repeat: [], rulename: \"c-wsp\"]},\n      {:rulename, \"alternation\"},\n      {:repetition, [repeat: [], rulename: \"c-wsp\"]},\n      \")\"\n    ]\n  ],\n  rule: [\n    rulename: \"option\",\n    concatenation: [\n      \"[\",\n      {:repetition, [repeat: [], rulename: \"c-wsp\"]},\n      {:rulename, \"alternation\"},\n      {:repetition, [repeat: [], rulename: \"c-wsp\"]},\n      \"]\"\n    ]\n  ],\n  rule: [\n    rulename: \"char-val\",\n    concatenation: [\n      rulename: \"DQUOTE\",\n      repetition: [\n        repeat: [],\n        alternation: [\n          num_range: [{:base, \"x\"}, \"20\", \"21\"],\n          num_range: [{:base, \"x\"}, \"23\", \"7E\"]\n        ]\n      ],\n      rulename: \"DQUOTE\"\n    ],\n    comment: \"quoted string of SP and VCHAR\",\n    comment: \"without DQUOTE\"\n  ],\n  rule: [\n    rulename: \"num-val\",\n    concatenation: [\n      \"%\",\n      {:alternation,\n       [rulename: \"bin-val\", rulename: \"dec-val\", rulename: \"hex-val\"]}\n    ]\n  ],\n  rule: [\n    rulename: \"bin-val\",\n    concatenation: [\n      \"b\",\n      {:repetition, [repeat: [min: 1], rulename: \"BIT\"]},\n      {:option,\n       [\n         alternation: [\n           repetition: [\n             repeat: [min: 1],\n             concatenation: [\n               \".\",\n               {:repetition, [repeat: [min: 1], rulename: \"BIT\"]}\n             ]\n           ],\n           concatenation: [\n             \"-\",\n             {:repetition, [repeat: [min: 1], rulename: \"BIT\"]}\n           ]\n         ]\n       ]}\n    ],\n    comment: \"series of concatenated bit values\",\n    comment: \"or single ONEOF range\"\n  ],\n  rule: [\n    rulename: \"dec-val\",\n    concatenation: [\n      \"d\",\n      {:repetition, [repeat: [min: 1], rulename: \"DIGIT\"]},\n      {:option,\n       [\n         alternation: [\n           repetition: [\n             repeat: [min: 1],\n             concatenation: [\n               \".\",\n               {:repetition, [repeat: [min: 1], rulename: \"DIGIT\"]}\n             ]\n           ],\n           concatenation: [\n             \"-\",\n             {:repetition, [repeat: [min: 1], rulename: \"DIGIT\"]}\n           ]\n         ]\n       ]}\n    ]\n  ],\n  rule: [\n    rulename: \"hex-val\",\n    concatenation: [\n      \"x\",\n      {:repetition, [repeat: [min: 1], rulename: \"HEXDIG\"]},\n      {:option,\n       [\n         alternation: [\n           repetition: [\n             repeat: [min: 1],\n             concatenation: [\n               \".\",\n               {:repetition, [repeat: [min: 1], rulename: \"HEXDIG\"]}\n             ]\n           ],\n           concatenation: [\n             \"-\",\n             {:repetition, [repeat: [min: 1], rulename: \"HEXDIG\"]}\n           ]\n         ]\n       ]}\n    ]\n  ],\n  rule: [\n    rulename: \"prose-val\",\n    concatenation: [\n      \"<\",\n      {:repetition,\n       [\n         repeat: [],\n         alternation: [\n           num_range: [{:base, \"x\"}, \"20\", \"3D\"],\n           num_range: [{:base, \"x\"}, \"3F\", \"7E\"]\n         ]\n       ]},\n      \">\"\n    ],\n    comment: \"bracketed string of SP and VCHAR\",\n    comment: \"without angles\",\n    comment: \"prose description, to be used as\",\n    comment: \"last resort\"\n  ]\n]\n```\n\nAnd generated parser looks like:\n\n```elixir\n[\n  defparsec(\n    :rulelist,\n    tag(\n      times(choice([parsec(:rule), repeat(parsec(:c_wsp)) |> parsec(:c_nl)]), min: 1),\n      :rulelist\n    )\n  ),\n  defparsec(\n    :rule,\n    tag(parsec(:rulename) |> parsec(:defined_as) |> parsec(:elements) |> parsec(:c_nl), :rule)\n  ),\n  defparsec(\n    :rulename,\n    tag(\n      parsec(:core_alpha)\n      |> repeat(choice([parsec(:core_alpha), parsec(:core_digit), string(\"-\")])),\n      :rulename\n    )\n  ),\n  defparsec(\n    :defined_as,\n    tag(\n      repeat(parsec(:c_wsp)) |> choice([string(\"=\"), string(\"=/\")]) |> repeat(parsec(:c_wsp)),\n      :defined_as\n    )\n  ),\n  defparsec(:elements, tag(parsec(:alternation) |> repeat(parsec(:c_wsp)), :elements)),\n  defparsec(:c_wsp, tag(choice([parsec(:core_wsp), parsec(:c_nl) |> parsec(:core_wsp)]), :c_wsp)),\n  defparsec(:c_nl, tag(choice([parsec(:comment), parsec(:core_crlf)]), :c_nl)),\n  defparsec(\n    :comment,\n    tag(\n      string(\";\")\n      |> repeat(choice([parsec(:core_wsp), parsec(:core_vchar)]))\n      |> parsec(:core_crlf),\n      :comment\n    )\n  ),\n  defparsec(\n    :alternation,\n    tag(\n      parsec(:concatenation)\n      |> repeat(\n        repeat(parsec(:c_wsp))\n        |> string(\"/\")\n        |> repeat(parsec(:c_wsp))\n        |> parsec(:concatenation)\n      ),\n      :alternation\n    )\n  ),\n  defparsec(\n    :concatenation,\n    tag(\n      parsec(:repetition) |> repeat(times(parsec(:c_wsp), min: 1) |> parsec(:repetition)),\n      :concatenation\n    )\n  ),\n  defparsec(:repetition, tag(optional(parsec(:repeat)) |> parsec(:element), :repetition)),\n  defparsec(\n    :repeat,\n    tag(\n      choice([\n        times(parsec(:core_digit), min: 1),\n        repeat(parsec(:core_digit)) |> string(\"*\") |> repeat(parsec(:core_digit))\n      ]),\n      :repeat\n    )\n  ),\n  defparsec(\n    :element,\n    tag(\n      choice([\n        parsec(:rulename),\n        parsec(:group),\n        parsec(:option),\n        parsec(:char_val),\n        parsec(:num_val),\n        parsec(:prose_val)\n      ]),\n      :element\n    )\n  ),\n  defparsec(\n    :group,\n    tag(\n      string(\"(\")\n      |> repeat(parsec(:c_wsp))\n      |> parsec(:alternation)\n      |> repeat(parsec(:c_wsp))\n      |> string(\")\"),\n      :group\n    )\n  ),\n  defparsec(\n    :option,\n    tag(\n      string(\"[\")\n      |> repeat(parsec(:c_wsp))\n      |> parsec(:alternation)\n      |> repeat(parsec(:c_wsp))\n      |> string(\"]\"),\n      :option\n    )\n  ),\n  defparsec(\n    :char_val,\n    tag(\n      parsec(:core_dquote)\n      |> repeat(choice([ascii_char([32..33]), ascii_char([35..126])]))\n      |> parsec(:core_dquote),\n      :char_val\n    )\n  ),\n  defparsec(\n    :num_val,\n    tag(string(\"%\") |> choice([parsec(:bin_val), parsec(:dec_val), parsec(:hex_val)]), :num_val)\n  ),\n  defparsec(\n    :bin_val,\n    tag(\n      string(\"b\")\n      |> times(parsec(:core_bit), min: 1)\n      |> optional(\n        choice([\n          times(string(\".\") |> times(parsec(:core_bit), min: 1), min: 1),\n          string(\"-\") |> times(parsec(:core_bit), min: 1)\n        ])\n      ),\n      :bin_val\n    )\n  ),\n  defparsec(\n    :dec_val,\n    tag(\n      string(\"d\")\n      |> times(parsec(:core_digit), min: 1)\n      |> optional(\n        choice([\n          times(string(\".\") |> times(parsec(:core_digit), min: 1), min: 1),\n          string(\"-\") |> times(parsec(:core_digit), min: 1)\n        ])\n      ),\n      :dec_val\n    )\n  ),\n  defparsec(\n    :hex_val,\n    tag(\n      string(\"x\")\n      |> times(parsec(:core_hexdig), min: 1)\n      |> optional(\n        choice([\n          times(string(\".\") |> times(parsec(:core_hexdig), min: 1), min: 1),\n          string(\"-\") |> times(parsec(:core_hexdig), min: 1)\n        ])\n      ),\n      :hex_val\n    )\n  ),\n  defparsec(\n    :prose_val,\n    tag(\n      string(\"<\") |> repeat(choice([ascii_char([32..61]), ascii_char([63..126])])) |> string(\">\"),\n      :prose_val\n    )\n  )\n]\n```","ref":"readme.html#what-does-it-do-really"}],"proglang":"elixir","content_type":"text/markdown","producer":{"name":"ex_doc","version":"0.36.1"}}